/*@lineinfo:filename=CollectionExample1*//*@lineinfo:user-code*//*@lineinfo:1^1*//*
   The program CollectionExample1.sqlj illustrates how to retrieve,
   add and modify a VARRAY containing VARCHAR2 strings.
   This program uses the customers3 table that contains a column named
   addresses, which is a VARRAY that provides storage space for up to
   three VARCHAR2 strings.

*/

import java.sql.*;
import oracle.sqlj.runtime.Oracle;

public class CollectionExample1 {

  /*@lineinfo:generated-code*//*@lineinfo:15^3*/

//  ************************************************************
//  SQLJ iterator declaration:
//  ************************************************************

private static class CustomerIteratorClass 
extends sqlj.runtime.ref.ResultSetIterImpl
implements sqlj.runtime.NamedIterator
{
  public CustomerIteratorClass(sqlj.runtime.profile.RTResultSet resultSet) 
    throws java.sql.SQLException 
  {
    super(resultSet);
    idNdx = findColumn("id");
    first_nameNdx = findColumn("first_name");
    last_nameNdx = findColumn("last_name");
    addressesNdx = findColumn("addresses");
  }
  public int id() 
    throws java.sql.SQLException 
  {
    return resultSet.getIntNoNull(idNdx);
  }
  private int idNdx;
  public String first_name() 
    throws java.sql.SQLException 
  {
    return resultSet.getString(first_nameNdx);
  }
  private int first_nameNdx;
  public String last_name() 
    throws java.sql.SQLException 
  {
    return resultSet.getString(last_nameNdx);
  }
  private int last_nameNdx;
  public TAddress2 addresses() 
    throws java.sql.SQLException 
  {
    return ((TAddress2)(resultSet.getObject(addressesNdx, TAddress2.class)));
  }
  private int addressesNdx;
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:17^3*/

  public static void main(String [] args) {

    try {

      Oracle.connect(
        "jdbc:oracle:thin:@localhost:1521:orcl",
        "object_user",
        "object_password"
      );

      displayCustomers();

      // create an array of strings to store the addresses
      String [] address_strings = new String[3];

      // populate the array with strings representing the addresses
      address_strings[0] = "1 Any Street, Any Town, CA, 12345";
      address_strings[1] = "2 Liberty Street, Uptown, CA, 54321";
      address_strings[2] = "17 Main Street, New Town, CA, 51234";

      // create a host object named addresses and populate the object
      // by passing the address_strings array to the constructor
      TAddress2 addresses = new TAddress2(address_strings);

      // add a new customer to the table
      addCustomer(3, "Jerry", "Fieldtop", addresses);

      // modify the second string in addresses using the setElement() method
      addresses.setElement("10 New Street, Big Town, CA, 12345", 1);
      /*@lineinfo:generated-code*//*@lineinfo:48^7*/

//  ************************************************************
//  #sql { UPDATE
//            customers3
//          SET
//            addresses = :addresses
//          WHERE
//            id = 3
//         };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  TAddress2 __sJT_1 = addresses;
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, CollectionExample1_SJProfileKeys.getKey(0), 0);
    try 
    {
      __sJT_stmt.setObject(1, __sJT_1);
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:55^7*/

      // use alternate method for populating the addresses host object
      // using the setArray() method
      addresses.setArray(address_strings);

      // add a new customer
      addCustomer(4, "Steve", "Edwards", addresses);

      displayCustomers();

      // rollback the changes and disconnect from the database
      /*@lineinfo:generated-code*//*@lineinfo:67^7*/

//  ************************************************************
//  #sql { ROLLBACK  };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, CollectionExample1_SJProfileKeys.getKey(0), 1);
    try 
    {
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:67^23*/
      Oracle.close();

    } catch (SQLException e) {

      System.err.println("SQLException " + e);
      System.exit(1);

    }

  } // end of main()


  // displayCustomers() selects and displays the customers
  private static void displayCustomers()
  throws SQLException {

    CustomerIteratorClass customer_iterator;

    /*@lineinfo:generated-code*//*@lineinfo:86^5*/

//  ************************************************************
//  #sql customer_iterator = { SELECT
//          id, first_name, last_name, addresses
//        FROM
//          customers3
//        ORDER BY
//          id
//       };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, CollectionExample1_SJProfileKeys.getKey(0), 2);
    try 
    {
      sqlj.runtime.profile.RTResultSet __sJT_result = __sJT_execCtx.executeQuery();
      customer_iterator = new CustomerIteratorClass(__sJT_result);
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:93^5*/

    System.out.println("List of all customers ...");

    while (customer_iterator.next()) {

      System.out.println("Customer:");
      System.out.println("id = " + customer_iterator.id());
      System.out.println("first_name = " + customer_iterator.first_name());
      System.out.println("last_name = " + customer_iterator.last_name());

      // get the total number of elements using the length() method
      // through addresses()
      int address_count = customer_iterator.addresses().length();

      // retrieve the strings stored in the host object using the
      // getArray() method through addresses()
      String [] address_array = customer_iterator.addresses().getArray();

      // display the address strings
      System.out.println("Total number of addresses = " + address_count);
      for (int counter = 0; counter < address_count; counter++) {
        System.out.println("Address " + counter + ":");
        System.out.println("Entire address = " + address_array[counter]);
      }

      // retrieve a single string stored in the host object using the
      // getElement() method through addresses()
      String address = customer_iterator.addresses().getElement(0);
      System.out.println("First address: " + address);

    } // end of while loop

    customer_iterator.close();

  } // end of displayCustomers()


  private static void addCustomer (

    int id,
    String first_name,
    String last_name,
    TAddress2 addresses

  ) throws SQLException {

    /*@lineinfo:generated-code*//*@lineinfo:140^5*/

//  ************************************************************
//  #sql { INSERT INTO
//          customers3 (
//            id, first_name, last_name, addresses
//          ) VALUES (
//            :id, :first_name, :last_name, :addresses
//          )
//       };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  int __sJT_1 = id;
  String __sJT_2 = first_name;
  String __sJT_3 = last_name;
  TAddress2 __sJT_4 = addresses;
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, CollectionExample1_SJProfileKeys.getKey(0), 3);
    try 
    {
      __sJT_stmt.setInt(1, __sJT_1);
      __sJT_stmt.setString(2, __sJT_2);
      __sJT_stmt.setString(3, __sJT_3);
      __sJT_stmt.setObject(4, __sJT_4);
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:147^5*/

  } // end of addCustomer()

}/*@lineinfo:generated-code*/class CollectionExample1_SJProfileKeys 
{
  private static CollectionExample1_SJProfileKeys inst = null;
  public static java.lang.Object getKey(int keyNum) 
    throws java.sql.SQLException 
  {
    if (inst == null)
    {
      inst = new CollectionExample1_SJProfileKeys();
    }
    return inst.keys[keyNum];
  }
  private final sqlj.runtime.profile.Loader loader = sqlj.runtime.RuntimeContext.getRuntime().getLoaderForClass(getClass());
  private java.lang.Object[] keys;
  private CollectionExample1_SJProfileKeys() 
    throws java.sql.SQLException 
  {
    keys = new java.lang.Object[1];
    keys[0] = sqlj.runtime.ref.DefaultContext.getProfileKey(loader, "CollectionExample1_SJProfile0");
  }
}
