/*@lineinfo:filename=CollectionExample2*//*@lineinfo:user-code*//*@lineinfo:1^1*//*
   The program CollectionExample2.sqlj illustrates how to retrieve,
   add and modify a VARRAY containing t_address objects.
   This program uses the customers4 table that contains a column named
   addresses, which is a VARRAY that provides storage space for up to
   three t_address objects.
*/

import java.sql.*;
import oracle.sqlj.runtime.Oracle;

public class CollectionExample2 {

  /*@lineinfo:generated-code*//*@lineinfo:14^3*/

//  ************************************************************
//  SQLJ iterator declaration:
//  ************************************************************

private static class CustomerIteratorClass 
extends sqlj.runtime.ref.ResultSetIterImpl
implements sqlj.runtime.NamedIterator
{
  public CustomerIteratorClass(sqlj.runtime.profile.RTResultSet resultSet) 
    throws java.sql.SQLException 
  {
    super(resultSet);
    idNdx = findColumn("id");
    first_nameNdx = findColumn("first_name");
    last_nameNdx = findColumn("last_name");
    addressesNdx = findColumn("addresses");
  }
  public int id() 
    throws java.sql.SQLException 
  {
    return resultSet.getIntNoNull(idNdx);
  }
  private int idNdx;
  public String first_name() 
    throws java.sql.SQLException 
  {
    return resultSet.getString(first_nameNdx);
  }
  private int first_nameNdx;
  public String last_name() 
    throws java.sql.SQLException 
  {
    return resultSet.getString(last_nameNdx);
  }
  private int last_nameNdx;
  public TAddress3 addresses() 
    throws java.sql.SQLException 
  {
    return ((TAddress3)(resultSet.getObject(addressesNdx, TAddress3.class)));
  }
  private int addressesNdx;
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:16^3*/


  public static void main(String [] args) {

    try {

      Oracle.connect(
        "jdbc:oracle:thin:@localhost:1521:orcl",
        "object_user",
        "object_password"
      );

      displayCustomers();

      // create an array of TAddress objects to store the addresses
      TAddress [] address_objects = new TAddress[3];
      address_objects[0] = new TAddress();
      address_objects[0].setStreet("1 Any Street");
      address_objects[0].setCity("Any Town");
      address_objects[0].setState("CA");
      address_objects[0].setZip("12345");
      address_objects[1] = new TAddress();
      address_objects[1].setStreet("2 Main Street");
      address_objects[1].setCity("Uptown");
      address_objects[1].setState("CA");
      address_objects[1].setZip("54321");
      address_objects[2] = new TAddress();
      address_objects[2].setStreet("7 Winpole Ave");
      address_objects[2].setCity("New Town");
      address_objects[2].setState("CA");
      address_objects[2].setZip("51234");

      // create a host object named addresses and populate the object
      // by passing the address_objects array to the constructor
      TAddress3 addresses = new TAddress3();
      addresses.setArray(address_objects);

      // add a new customer
      addCustomer(3, "Jerry", "Fieldtop", addresses);

      // modify the second object in address_array using the setElement() method
      address_objects[1].setStreet("1 Brown Street");
      address_objects[1].setCity("Uptown");
      address_objects[1].setState("CA");
      address_objects[1].setZip("54321");
      addresses.setElement(address_objects[1], 1);
      /*@lineinfo:generated-code*//*@lineinfo:63^7*/

//  ************************************************************
//  #sql { UPDATE
//            customers4
//          SET
//            addresses = :addresses
//          WHERE
//            id = 3
//         };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  TAddress3 __sJT_1 = addresses;
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, CollectionExample2_SJProfileKeys.getKey(0), 0);
    try 
    {
      __sJT_stmt.setObject(1, __sJT_1);
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:70^7*/

      // use alternate method for populating the addresses host object
      // using the setArray() method
      addresses.setArray(address_objects);

      // add a new customer
      addCustomer(4, "Steve", "Edwards", addresses);

      displayCustomers();

      // rollback the changes and disconnect from the database
      /*@lineinfo:generated-code*//*@lineinfo:82^7*/

//  ************************************************************
//  #sql { ROLLBACK  };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, CollectionExample2_SJProfileKeys.getKey(0), 1);
    try 
    {
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:82^23*/
      Oracle.close();

    } catch (SQLException e) {

      System.err.println("SQLException " + e);
      System.exit(1);

    }

  } // end of main()


  // displayCustomers selects and displays the customers
  private static void displayCustomers()
  throws SQLException {

    CustomerIteratorClass customer_iterator;

    /*@lineinfo:generated-code*//*@lineinfo:101^5*/

//  ************************************************************
//  #sql customer_iterator = { SELECT
//          id, first_name, last_name, addresses
//        FROM
//          customers4
//        ORDER BY
//          id
//       };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, CollectionExample2_SJProfileKeys.getKey(0), 2);
    try 
    {
      sqlj.runtime.profile.RTResultSet __sJT_result = __sJT_execCtx.executeQuery();
      customer_iterator = new CustomerIteratorClass(__sJT_result);
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:108^5*/

    System.out.println("List of all customers ...");

    while (customer_iterator.next()) {

      System.out.println("Customer:");
      System.out.println("id = " + customer_iterator.id());
      System.out.println("first_name = " + customer_iterator.first_name());
      System.out.println("last_name = " + customer_iterator.last_name());

      // get the total number of elements using the length() method
      // through addresses()
      int address_count = customer_iterator.addresses().length();

      // retrieve the address objects stored in the host object using the
      // getArray() method through addresses()
      TAddress [] address_array = customer_iterator.addresses().getArray();

      // display the address objects
      System.out.println("Total number of addresses = " + address_count);
      for (int counter = 0; counter < address_count; counter++) {
        System.out.println("Address " + counter + ":");
        System.out.println("street = " + address_array[counter].getStreet());
        System.out.println("city = " + address_array[counter].getCity());
        System.out.println("state = " + address_array[counter].getState());
        System.out.println("zip = " + address_array[counter].getZip());
      }
    } // end of while loop

    // close the iterator
    customer_iterator.close();

  } // end of displayCustomers()


  private static void addCustomer (

    int id,
    String first_name,
    String last_name,
    TAddress3 addresses

  ) throws SQLException {

    /*@lineinfo:generated-code*//*@lineinfo:153^5*/

//  ************************************************************
//  #sql { INSERT INTO
//          customers4 (
//            id, first_name, last_name, addresses
//          ) VALUES (
//            :id, :first_name, :last_name, :addresses
//          )
//       };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  int __sJT_1 = id;
  String __sJT_2 = first_name;
  String __sJT_3 = last_name;
  TAddress3 __sJT_4 = addresses;
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, CollectionExample2_SJProfileKeys.getKey(0), 3);
    try 
    {
      __sJT_stmt.setInt(1, __sJT_1);
      __sJT_stmt.setString(2, __sJT_2);
      __sJT_stmt.setString(3, __sJT_3);
      __sJT_stmt.setObject(4, __sJT_4);
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:160^5*/

  } // end of addCustomer()

}/*@lineinfo:generated-code*/class CollectionExample2_SJProfileKeys 
{
  private static CollectionExample2_SJProfileKeys inst = null;
  public static java.lang.Object getKey(int keyNum) 
    throws java.sql.SQLException 
  {
    if (inst == null)
    {
      inst = new CollectionExample2_SJProfileKeys();
    }
    return inst.keys[keyNum];
  }
  private final sqlj.runtime.profile.Loader loader = sqlj.runtime.RuntimeContext.getRuntime().getLoaderForClass(getClass());
  private java.lang.Object[] keys;
  private CollectionExample2_SJProfileKeys() 
    throws java.sql.SQLException 
  {
    keys = new java.lang.Object[1];
    keys[0] = sqlj.runtime.ref.DefaultContext.getProfileKey(loader, "CollectionExample2_SJProfile0");
  }
}
