/*@lineinfo:filename=ContextExample1*//*@lineinfo:user-code*//*@lineinfo:1^1*//*
   The program ContextExample1.sqlj illustrates how to use
   DefaultContext connection context objects to make multiple
   connections to a database.
*/

import java.sql.*;
import oracle.sqlj.runtime.Oracle;
import sqlj.runtime.ref.DefaultContext;

public class ContextExample1 {

  // declare the iterator class
  /*@lineinfo:generated-code*//*@lineinfo:14^3*/

//  ************************************************************
//  SQLJ iterator declaration:
//  ************************************************************

private static class CustomerIteratorClass 
extends sqlj.runtime.ref.ResultSetIterImpl
implements sqlj.runtime.NamedIterator
{
  public CustomerIteratorClass(sqlj.runtime.profile.RTResultSet resultSet) 
    throws java.sql.SQLException 
  {
    super(resultSet);
    idNdx = findColumn("id");
    first_nameNdx = findColumn("first_name");
    last_nameNdx = findColumn("last_name");
  }
  public int id() 
    throws java.sql.SQLException 
  {
    return resultSet.getIntNoNull(idNdx);
  }
  private int idNdx;
  public String first_name() 
    throws java.sql.SQLException 
  {
    return resultSet.getString(first_nameNdx);
  }
  private int first_nameNdx;
  public String last_name() 
    throws java.sql.SQLException 
  {
    return resultSet.getString(last_nameNdx);
  }
  private int last_nameNdx;
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:16^3*/

  public static void main(String [] args) {

    try {

      // set the default connection context using the Oracle.connect()
      // method
      Oracle.connect(
        "jdbc:oracle:thin:@localhost:1521:orcl",
        "fundamental_user",
        "fundamental_password"
      );

      // connect to database using a second connection context
      // named conn_context
      DefaultContext conn_context = Oracle.getConnection(
        "jdbc:oracle:thin:@localhost:1521:orcl",
        "fundamental_user",
        "fundamental_password"
      );

      // add a row to the customers table using conn_context
      System.out.println("Adding customer Jason Price using " +
        "conn_context.");
      /*@lineinfo:generated-code*//*@lineinfo:41^7*/

//  ************************************************************
//  #sql [conn_context] { INSERT INTO
//            customers (id, first_name, last_name)
//          VALUES
//            (6, 'Jason', 'Price')
//         };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = conn_context;
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, ContextExample1_SJProfileKeys.getKey(0), 0);
    try 
    {
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:46^7*/

      // update the first row in the customers table using conn_context
      System.out.println("Updating customer 1 name to John Doe using " +
        "conn_context.");
      /*@lineinfo:generated-code*//*@lineinfo:51^7*/

//  ************************************************************
//  #sql [conn_context] { UPDATE
//            customers
//          SET
//            first_name = 'John',
//            last_name  = 'Doe'
//          WHERE
//            id = 1
//         };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = conn_context;
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, ContextExample1_SJProfileKeys.getKey(0), 1);
    try 
    {
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:59^7*/

      // display all rows in the customers table using the default
      // connection context, the new row and the update are
      // not visible to the default context because the row was added
      // using conn_context (which has a separate database transaction
      // associated with it)
      displayCustomers();

      // switch the default context to conn_context
      System.out.println("Switching default connection context " +
        "to conn_context.");
      DefaultContext.setDefaultContext(conn_context);

      // display all the rows in the customers table again using
      // the default connection context, the changes are now visible
      // because the default context has been switched to conn_context
      displayCustomers();

      // rollback the changes
      /*@lineinfo:generated-code*//*@lineinfo:79^7*/

//  ************************************************************
//  #sql { ROLLBACK  };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, ContextExample1_SJProfileKeys.getKey(0), 2);
    try 
    {
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:79^23*/

      // close the conn_context connection context
      conn_context.close();

      // close the default connection context
      Oracle.close();

    } catch ( SQLException e ) {

      System.err.println("SQLException " + e);
      System.exit(1);

    }

  } // end of main()


  private static void displayCustomers()
  throws SQLException {

    // declare a named iterator object
    CustomerIteratorClass customer_iterator;

    // use the default context when populating the iterator
    /*@lineinfo:generated-code*//*@lineinfo:104^5*/

//  ************************************************************
//  #sql customer_iterator = { SELECT
//          id, first_name, last_name
//        FROM
//          customers
//        ORDER BY
//          id
//       };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, ContextExample1_SJProfileKeys.getKey(0), 3);
    try 
    {
      sqlj.runtime.profile.RTResultSet __sJT_result = __sJT_execCtx.executeQuery();
      customer_iterator = new CustomerIteratorClass(__sJT_result);
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:111^5*/

    System.out.println("List of customers using default connection " + 
      "context.");

    // access the contents of the iterator
    while (customer_iterator.next()) {

      // display the customer
      System.out.println("Customer:");
      System.out.println("id = " + customer_iterator.id());
      System.out.println("first_name = " + customer_iterator.first_name());
      System.out.println("last_name = " + customer_iterator.last_name());

    } // end of while loop

    // close the iterator
    customer_iterator.close();

  } // end of displayCustomers()

}/*@lineinfo:generated-code*/class ContextExample1_SJProfileKeys 
{
  private static ContextExample1_SJProfileKeys inst = null;
  public static java.lang.Object getKey(int keyNum) 
    throws java.sql.SQLException 
  {
    if (inst == null)
    {
      inst = new ContextExample1_SJProfileKeys();
    }
    return inst.keys[keyNum];
  }
  private final sqlj.runtime.profile.Loader loader = sqlj.runtime.RuntimeContext.getRuntime().getLoaderForClass(getClass());
  private java.lang.Object[] keys;
  private ContextExample1_SJProfileKeys() 
    throws java.sql.SQLException 
  {
    keys = new java.lang.Object[1];
    keys[0] = DefaultContext.getProfileKey(loader, "ContextExample1_SJProfile0");
  }
}
