/*@lineinfo:filename=ContextExample2*//*@lineinfo:user-code*//*@lineinfo:1^1*//*
   The program ContextExample2.sqlj illustrates how to create
   and use execution context objects.
*/

import java.sql.*;
import oracle.sqlj.runtime.Oracle;
import sqlj.runtime.ref.DefaultContext;
import sqlj.runtime.ExecutionContext;

public class ContextExample2 {

  // declare the iterator class
  /*@lineinfo:generated-code*//*@lineinfo:14^3*/

//  ************************************************************
//  SQLJ iterator declaration:
//  ************************************************************

private static class CustomerIteratorClass 
extends sqlj.runtime.ref.ResultSetIterImpl
implements sqlj.runtime.NamedIterator
{
  public CustomerIteratorClass(sqlj.runtime.profile.RTResultSet resultSet) 
    throws java.sql.SQLException 
  {
    super(resultSet);
    idNdx = findColumn("id");
    first_nameNdx = findColumn("first_name");
    last_nameNdx = findColumn("last_name");
  }
  public int id() 
    throws java.sql.SQLException 
  {
    return resultSet.getIntNoNull(idNdx);
  }
  private int idNdx;
  public String first_name() 
    throws java.sql.SQLException 
  {
    return resultSet.getString(first_nameNdx);
  }
  private int first_nameNdx;
  public String last_name() 
    throws java.sql.SQLException 
  {
    return resultSet.getString(last_nameNdx);
  }
  private int last_nameNdx;
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:16^3*/

  public static void main(String [] args) {

    try {

      // connect to database using a connection context
      // named conn_context
      DefaultContext conn_context = Oracle.getConnection(
        "jdbc:oracle:thin:@localhost:1521:orcl",
        "fundamental_user",
        "fundamental_password"
      );

      // create an execution context named exec_context
      ExecutionContext exec_context = new ExecutionContext();

      // create a second execution context named exec_context2
      ExecutionContext exec_context2 = new ExecutionContext();

      // display all customers using conn_context and exec_context
      System.out.println("List of customers using connection context " +
        "conn_context and execution context exec_context.");
      displayCustomers(conn_context, exec_context);

      // update customer using conn_context and exec_context
      System.out.println("Updating customer 1 name to John Doe using " +
        "conn_context and exec_context.");
      /*@lineinfo:generated-code*//*@lineinfo:44^7*/

//  ************************************************************
//  #sql [conn_context, exec_context] { UPDATE
//            customers
//          SET
//            first_name = 'John',
//            last_name  = 'Doe'
//          WHERE
//            id = 1
//         };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = conn_context;
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx =  exec_context;
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, ContextExample2_SJProfileKeys.getKey(0), 0);
    try 
    {
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:52^7*/

      // update customer using conn_context and exec_context2,
      System.out.println("Updating customer 1 name to Jean Smith using " +
        "conn_context and exec_context2.");
      /*@lineinfo:generated-code*//*@lineinfo:57^7*/

//  ************************************************************
//  #sql [conn_context, exec_context2] { UPDATE
//            customers
//          SET
//            first_name = 'Jean',
//            last_name  = 'Smith'
//          WHERE
//            id = 1
//         };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = conn_context;
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx =  exec_context2;
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, ContextExample2_SJProfileKeys.getKey(0), 1);
    try 
    {
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:65^7*/

      // display all customers using the conn_context and exec_context
      System.out.println("List of customers using conn_context " +
        "and exec_context.");
      displayCustomers(conn_context, exec_context);

      // perform an explicit ROLLBACK using conn_context and exec_context
      /*@lineinfo:generated-code*//*@lineinfo:73^7*/

//  ************************************************************
//  #sql [conn_context, exec_context] { ROLLBACK  };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = conn_context;
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx =  exec_context;
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, ContextExample2_SJProfileKeys.getKey(0), 2);
    try 
    {
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:73^52*/

      // close the connection context
      conn_context.close();

    } catch ( SQLException e ) {

      System.err.println("SQLException " + e);
      System.exit(1);

    }

  } // end of main()


  private static void displayCustomers(
    DefaultContext   conn_context,
    ExecutionContext exec_context
  ) throws SQLException {

    // instantiate a named iterator object
    CustomerIteratorClass customer_iterator;

    /*@lineinfo:generated-code*//*@lineinfo:96^5*/

//  ************************************************************
//  #sql [conn_context, exec_context] customer_iterator = { SELECT
//          id, first_name, last_name
//        FROM
//          customers
//        ORDER BY
//          id
//       };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = conn_context;
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx =  exec_context;
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, ContextExample2_SJProfileKeys.getKey(0), 3);
    try 
    {
      sqlj.runtime.profile.RTResultSet __sJT_result = __sJT_execCtx.executeQuery();
      customer_iterator = new CustomerIteratorClass(__sJT_result);
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:103^5*/

    // access the contents of the iterator
    while (customer_iterator.next()) {

      System.out.println("Customer:");
      System.out.println("id = " + customer_iterator.id());
      System.out.println("first_name = " + customer_iterator.first_name());
      System.out.println("last_name = " + customer_iterator.last_name());

    } // end of while loop

    // close the iterator
    customer_iterator.close();

  } // end of displayCustomers()

}/*@lineinfo:generated-code*/class ContextExample2_SJProfileKeys 
{
  private static ContextExample2_SJProfileKeys inst = null;
  public static java.lang.Object getKey(int keyNum) 
    throws java.sql.SQLException 
  {
    if (inst == null)
    {
      inst = new ContextExample2_SJProfileKeys();
    }
    return inst.keys[keyNum];
  }
  private final sqlj.runtime.profile.Loader loader = sqlj.runtime.RuntimeContext.getRuntime().getLoaderForClass(getClass());
  private java.lang.Object[] keys;
  private ContextExample2_SJProfileKeys() 
    throws java.sql.SQLException 
  {
    keys = new java.lang.Object[1];
    keys[0] = DefaultContext.getProfileKey(loader, "ContextExample2_SJProfile0");
  }
}
