/*@lineinfo:filename=FundamentalExample5*//*@lineinfo:user-code*//*@lineinfo:1^1*//*
   The program FundamentalExample5 illustrates the use of
   using nested cursors with positional iterators.
*/

// import required packages
import java.sql.*;
import oracle.sqlj.runtime.Oracle;

public class FundamentalExample5 {

  // declare the iterator classes
  /*@lineinfo:generated-code*//*@lineinfo:13^3*/

//  ************************************************************
//  SQLJ iterator declaration:
//  ************************************************************

public static class ProductDetailPosIteratorClass 
extends sqlj.runtime.ref.ResultSetIterImpl
implements sqlj.runtime.PositionedIterator
{
  public ProductDetailPosIteratorClass(sqlj.runtime.profile.RTResultSet resultSet) 
    throws java.sql.SQLException 
  {
    super(resultSet, 2);
  }
  public String getCol1() 
    throws java.sql.SQLException 
  {
    return resultSet.getString(1);
  }
  public double getCol2() 
    throws java.sql.SQLException 
  {
    return resultSet.getDoubleNoNull(2);
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:16^3*/
  /*@lineinfo:generated-code*//*@lineinfo:17^3*/

//  ************************************************************
//  SQLJ iterator declaration:
//  ************************************************************

private static class ProductMasterPosIteratorClass 
extends sqlj.runtime.ref.ResultSetIterImpl
implements sqlj.runtime.PositionedIterator
{
  public ProductMasterPosIteratorClass(sqlj.runtime.profile.RTResultSet resultSet) 
    throws java.sql.SQLException 
  {
    super(resultSet, 2);
  }
  public String getCol1() 
    throws java.sql.SQLException 
  {
    return resultSet.getString(1);
  }
  public ProductDetailPosIteratorClass getCol2() 
    throws java.sql.SQLException 
  {
    return ((ProductDetailPosIteratorClass)(resultSet.getObject(2, ProductDetailPosIteratorClass.class)));
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:20^3*/

  public static void main(String [] args) throws SQLException {

    try {

      Oracle.connect(
        "jdbc:oracle:thin:@localhost:1521:orcl",
        "fundamental_user",
        "fundamental_password"
      );

      // declare an iterator object to read the master rows
      ProductMasterPosIteratorClass product_master_iterator;

      // populate the master iterator object using a SELECT statement
      // containing the nested cursor
      /*@lineinfo:generated-code*//*@lineinfo:37^7*/

//  ************************************************************
//  #sql product_master_iterator = { SELECT
//            name, CURSOR (
//              SELECT
//                name, price
//              FROM
//                products
//              WHERE
//                products.type_id = product_types.id
//            )
//          FROM
//            product_types
//         };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, FundamentalExample5_SJProfileKeys.getKey(0), 0);
    try 
    {
      sqlj.runtime.profile.RTResultSet __sJT_result = __sJT_execCtx.executeQuery();
      product_master_iterator = new ProductMasterPosIteratorClass(__sJT_result);
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:49^7*/

      boolean finished = false;
      String product_type = null;
      ProductDetailPosIteratorClass product_detail_iterator = null;

      // read the master rows from the master iterator object
      while (!finished) {

        /*@lineinfo:generated-code*//*@lineinfo:58^9*/

//  ************************************************************
//  #sql { FETCH
//              :product_master_iterator
//            
//           };
//  ************************************************************

{
  ProductMasterPosIteratorClass __sJT_iter = product_master_iterator;
  if (__sJT_iter.next())
  {
    product_type = __sJT_iter.getCol1();
    product_detail_iterator = __sJT_iter.getCol2();
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:63^9*/

        if (product_master_iterator.endFetch()) {
          finished = true;
        } else {

          // display the product type
          System.out.println("Product type = " + product_type);

          boolean finished_inner = false;
          String product_name = null;
          double product_price = 0.0;

          // read the detail rows from the detail iterator object
          while (!finished_inner) {

            /*@lineinfo:generated-code*//*@lineinfo:79^13*/

//  ************************************************************
//  #sql { FETCH
//                  :product_detail_iterator
//                
//               };
//  ************************************************************

{
  ProductDetailPosIteratorClass __sJT_iter = product_detail_iterator;
  if (__sJT_iter.next())
  {
    product_name = __sJT_iter.getCol1();
    product_price = __sJT_iter.getCol2();
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:84^13*/

            if (product_detail_iterator.endFetch()) {
              finished_inner = true;
            } else {

              // display the product name and price
              System.out.println("  Product name = " + product_name +
                ", price = " + product_price);

            } // end of inner if

          } // end of inner while loop

          // close the detail iterator object
          product_detail_iterator.close();

        } // end of outer if

      } // end of outer while loop

      // close the master iterator object
      product_master_iterator.close();

      Oracle.close();

    } catch ( SQLException e ) {

      System.err.println("SQLException " + e);
      System.exit(1);

    }

  } // end of main()

}/*@lineinfo:generated-code*/class FundamentalExample5_SJProfileKeys 
{
  private static FundamentalExample5_SJProfileKeys inst = null;
  public static java.lang.Object getKey(int keyNum) 
    throws java.sql.SQLException 
  {
    if (inst == null)
    {
      inst = new FundamentalExample5_SJProfileKeys();
    }
    return inst.keys[keyNum];
  }
  private final sqlj.runtime.profile.Loader loader = sqlj.runtime.RuntimeContext.getRuntime().getLoaderForClass(getClass());
  private java.lang.Object[] keys;
  private FundamentalExample5_SJProfileKeys() 
    throws java.sql.SQLException 
  {
    keys = new java.lang.Object[1];
    keys[0] = sqlj.runtime.ref.DefaultContext.getProfileKey(loader, "FundamentalExample5_SJProfile0");
  }
}
