/*@lineinfo:filename=JdbcSqljExample2*//*@lineinfo:user-code*//*@lineinfo:1^1*//*
  The program JdbcSqljExample2.sqlj illustrates the use of sharing
  rows between SQLJ iterators and JDBC result sets.
*/

import java.sql.*;
import oracle.sqlj.runtime.Oracle;
import sqlj.runtime.ref.DefaultContext;

public class JdbcSqljExample2 {

  // declare a named iterator class
  /*@lineinfo:generated-code*//*@lineinfo:13^3*/

//  ************************************************************
//  SQLJ iterator declaration:
//  ************************************************************

public static class CustomerIteratorClass 
extends sqlj.runtime.ref.ResultSetIterImpl
implements sqlj.runtime.NamedIterator
{
  public CustomerIteratorClass(sqlj.runtime.profile.RTResultSet resultSet) 
    throws java.sql.SQLException 
  {
    super(resultSet);
    idNdx = findColumn("id");
    first_nameNdx = findColumn("first_name");
    last_nameNdx = findColumn("last_name");
  }
  public int id() 
    throws java.sql.SQLException 
  {
    return resultSet.getIntNoNull(idNdx);
  }
  private int idNdx;
  public String first_name() 
    throws java.sql.SQLException 
  {
    return resultSet.getString(first_nameNdx);
  }
  private int first_nameNdx;
  public String last_name() 
    throws java.sql.SQLException 
  {
    return resultSet.getString(last_nameNdx);
  }
  private int last_nameNdx;
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:15^3*/

  public static void main(String [] args) {

    try {

      Oracle.connect(
        "jdbc:oracle:thin:@localhost:1521:orcl",
        "fundamental_user",
        "fundamental_password"
      );

      // declare a SQLJ iterator
      CustomerIteratorClass customer_iterator;

      // populate the iterator
      /*@lineinfo:generated-code*//*@lineinfo:31^7*/

//  ************************************************************
//  #sql customer_iterator = { SELECT
//            id, first_name, last_name
//          FROM
//            customers
//          ORDER BY
//            id
//         };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, JdbcSqljExample2_SJProfileKeys.getKey(0), 0);
    try 
    {
      sqlj.runtime.profile.RTResultSet __sJT_result = __sJT_execCtx.executeQuery();
      customer_iterator = new CustomerIteratorClass(__sJT_result);
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:38^7*/

      // declare a JDBC result set, and share the rows in the SQLJ iterator
      // with the result set using a call to getResultSet()
      ResultSet customer_result_set = customer_iterator.getResultSet();

      System.out.println("Customer rows using JDBC result set:");
      while (customer_result_set.next()) {

        System.out.println("id = " + customer_result_set.getInt(1));
        System.out.println("first name = " +
          customer_result_set.getString(2));
        System.out.println("last name = " +
          customer_result_set.getString(3));

      } // end of while loop

      // close the iterator
      customer_iterator.close();

      // create a JDBC connection
      Connection jdbc_conn =
        DefaultContext.getDefaultContext().getConnection();

      // create a string containing the SELECT statement
      String sql_statement = "SELECT id, first_name, last_name FROM " +
        "customers ORDER BY id";

      // create a JDBC statement
      Statement statement = jdbc_conn.createStatement();

      // declare and populate a JDBC result set with the retrieved
      // rows from the prepared statement
      ResultSet customer_result_set2 =
        statement.executeQuery(sql_statement);

      // declare a SQLJ iterator
      CustomerIteratorClass customer_iterator2;

      // share the rows in the JDBC result with the SQLJ iterator using
      // the SQLJ CAST statement
      /*@lineinfo:generated-code*//*@lineinfo:79^7*/

//  ************************************************************
//  #sql customer_iterator2 = { CAST :customer_result_set2  };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  ResultSet __sJT_2 = customer_result_set2;
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, JdbcSqljExample2_SJProfileKeys.getKey(0), 1);
    try 
    {
      __sJT_stmt.setObject(2, __sJT_2);
      __sJT_execCtx.executeUpdate();
      CustomerIteratorClass __sJT_result;
      __sJT_result = ((CustomerIteratorClass)(__sJT_stmt.getObject(1, CustomerIteratorClass.class)));
      customer_iterator2 = __sJT_result;
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:79^62*/

      System.out.println("Customer rows using SQLJ iterator:");
      while (customer_iterator2.next()) {

        System.out.println("id = " + customer_iterator2.id());
        System.out.println("first_name = " +
          customer_iterator2.first_name());
        System.out.println("last_name = " +
          customer_iterator2.last_name());

      } // end of while loop

      // close the iterator
      customer_iterator2.close();

      jdbc_conn.close();
      Oracle.close();

    } catch ( SQLException e ) {

      System.err.println("SQLException " + e);
      System.exit(1);

    }

  } // end of main()

}/*@lineinfo:generated-code*/class JdbcSqljExample2_SJProfileKeys 
{
  private static JdbcSqljExample2_SJProfileKeys inst = null;
  public static java.lang.Object getKey(int keyNum) 
    throws java.sql.SQLException 
  {
    if (inst == null)
    {
      inst = new JdbcSqljExample2_SJProfileKeys();
    }
    return inst.keys[keyNum];
  }
  private final sqlj.runtime.profile.Loader loader = sqlj.runtime.RuntimeContext.getRuntime().getLoaderForClass(getClass());
  private java.lang.Object[] keys;
  private JdbcSqljExample2_SJProfileKeys() 
    throws java.sql.SQLException 
  {
    keys = new java.lang.Object[1];
    keys[0] = sqlj.runtime.ref.DefaultContext.getProfileKey(loader, "JdbcSqljExample2_SJProfile0");
  }
}
