/*@lineinfo:filename=LobExample2*//*@lineinfo:user-code*//*@lineinfo:1^1*//*
   The program LobExample2.sqlj illustrates how to
   retrieve content from the large object types
   CLOB, BLOB and BFILE.
*/

import java.sql.*;
import oracle.sqlj.runtime.Oracle;
import java.io.*;
import sqlj.runtime.*;
import oracle.sql.*;

public class LobExample2 {

  public static void main(String [] args) {

    try {

      Oracle.connect(
        "jdbc:oracle:thin:@localhost:1521:orcl",
        "lob_user",
        "lob_password"
      );

      String source_directory = "C:\\sample_files\\";
      String target_directory = "C:\\sample_files\\retrieved\\";

      retrieveCLOB("textContent.txt", source_directory, target_directory);
      retrieveBLOB("imageContent.jpg", source_directory, target_directory);
      retrieveBFILE("textContent.txt", target_directory);
      retrieveBFILE("imageContent.jpg", target_directory);

      Oracle.close();

    } catch (IOException e) {

      System.err.println("IOException " + e);
      System.exit(1);

    } catch (SQLException e) {

      System.err.println("SQLException " + e);
      System.exit(1);

    }

  }


  private static void retrieveCLOB(
    String file_name,
    String source_directory,
    String target_directory
  ) throws IOException, SQLException {

    // step 1: declare a host object of the LOB type
    CLOB host_clob;

    // step 2: retrieve the LOB locator into the LOB host object
    /*@lineinfo:generated-code*//*@lineinfo:60^5*/

//  ************************************************************
//  #sql { SELECT
//          clob_column
//        
//        FROM
//          clob_content
//        WHERE
//          file_name = :source_directory + file_name
//       };
//  ************************************************************

{
  sqlj.runtime.profile.RTResultSet __sJT_rtRs;
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  String __sJT_1 = source_directory + file_name;
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, LobExample2_SJProfileKeys.getKey(0), 0);
    try 
    {
      __sJT_stmt.setString(1, __sJT_1);
      sqlj.runtime.profile.RTResultSet __sJT_result = __sJT_execCtx.executeQuery();
      __sJT_rtRs = __sJT_result;
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
  try 
  {
    sqlj.runtime.ref.ResultSetIterImpl.checkColumns(__sJT_rtRs, 1);
    if (!__sJT_rtRs.next())
    {
      sqlj.runtime.error.RuntimeRefErrors.raise_NO_ROW_SELECT_INTO();
    }
    host_clob = ((CLOB)(__sJT_rtRs.getObject(1, CLOB.class)));
    if (__sJT_rtRs.next())
    {
      sqlj.runtime.error.RuntimeRefErrors.raise_MULTI_ROW_SELECT_INTO();
    }
  }
  finally 
  {
    __sJT_rtRs.close();
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:69^5*/

    // step 3: get the chunk size of the LOB from the LOB host object
    int size_of_chunk = host_clob.getChunkSize();

    // step 4: create a buffer to hold a chunk of data retrieved from
    // the LOB host object
    char [] text_buffer = new char[size_of_chunk];

    // step 5: create a file object
    String save_file = target_directory + "retrievedCLOB" + file_name;
    File file = new File(save_file);

    // step 6: create output stream objects to write the LOB contents
    // to the new file
    FileOutputStream file_output_stream = new FileOutputStream(file);
    OutputStreamWriter writer = new OutputStreamWriter(file_output_stream);
    BufferedWriter buffered_writer = new BufferedWriter(writer);

    // step 7: get the length of the LOB contents from the LOB host object
    long clob_length = host_clob.length();

    // step 8: while the end of the LOB contents has not been reached,
    // read a chunk of data from the LOB into the buffer,
    // and write the buffer contents to the file
    for (
      long position = 1;
      position <= clob_length;
      position += size_of_chunk
    ) {

      // read a chunk of data from host_clob into the buffer using the
      // getChars() method
      int chars_read = host_clob.getChars(position, size_of_chunk,
        text_buffer);

      // write the buffer contents to the file
      buffered_writer.write(text_buffer);

    } // end of for

    // step 9: close the stream objects
    buffered_writer.close();
    writer.close();
    file_output_stream.close();

    System.out.println("Successfully retrieved CLOB and saved file " + save_file);

  } // end of retrieveCLOB()


  private static void retrieveBLOB(
    String file_name,
    String source_directory,
    String target_directory
  ) throws IOException, SQLException {

    // step 1: declare a host object of the LOB type
    BLOB host_blob;

    // step 2: retrieve the LOB locator into the LOB host object
    /*@lineinfo:generated-code*//*@lineinfo:130^5*/

//  ************************************************************
//  #sql { SELECT
//          blob_column
//        
//        FROM
//          blob_content
//        WHERE
//          file_name = :source_directory + file_name
//       };
//  ************************************************************

{
  sqlj.runtime.profile.RTResultSet __sJT_rtRs;
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  String __sJT_1 = source_directory + file_name;
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, LobExample2_SJProfileKeys.getKey(0), 1);
    try 
    {
      __sJT_stmt.setString(1, __sJT_1);
      sqlj.runtime.profile.RTResultSet __sJT_result = __sJT_execCtx.executeQuery();
      __sJT_rtRs = __sJT_result;
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
  try 
  {
    sqlj.runtime.ref.ResultSetIterImpl.checkColumns(__sJT_rtRs, 1);
    if (!__sJT_rtRs.next())
    {
      sqlj.runtime.error.RuntimeRefErrors.raise_NO_ROW_SELECT_INTO();
    }
    host_blob = ((BLOB)(__sJT_rtRs.getObject(1, BLOB.class)));
    if (__sJT_rtRs.next())
    {
      sqlj.runtime.error.RuntimeRefErrors.raise_MULTI_ROW_SELECT_INTO();
    }
  }
  finally 
  {
    __sJT_rtRs.close();
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:139^5*/

    // step 3: get the chunk size of the LOB from the LOB host object
    int size_of_chunk = host_blob.getChunkSize();

    // step 4: create a buffer to hold a chunk of data retrieved from
    // the LOB host object
    byte [] byte_buffer = new byte[size_of_chunk];

    // step 5: create a file object
    String save_file_name = target_directory + "retrievedBLOB" + file_name;
    File file = new File(save_file_name);

    // step 6: create output stream object to write the LOB contents
    // to the new file
    FileOutputStream file_output_stream = new FileOutputStream(file);

    // step 7: get the length of the LOB contents from the LOB host object
    long blob_length = host_blob.length();

    // step 8: while the end of the LOB contents has not been reached,
    // read a chunk of data from the LOB into the buffer,
    // and write the buffer contents to the file
    for (
      long position = 1;
      position <= blob_length;
      position += size_of_chunk
    ) {

      // read a chunk of data from host_blob into the buffer using the
      // getBytes() method
      int bytes_read = host_blob.getBytes(position, size_of_chunk,
        byte_buffer);

      // write the buffer contents to the file
      file_output_stream.write(byte_buffer);

    } // end of for

    // step 9: close the stream objects
    file_output_stream.close();

    System.out.println("Successfully retrieved BLOB and saved file " +
      save_file_name);

  } // end of retrieveBLOB()


  private static void retrieveBFILE(
    String file_name,
    String target_directory
  ) throws IOException, SQLException {

    // step 1: declare a BFILE host object
    BFILE host_bfile;

    // step 2: retrieve the BFILE column into the BFILE host object
    /*@lineinfo:generated-code*//*@lineinfo:196^5*/

//  ************************************************************
//  #sql { SELECT
//          bfile_column
//        
//        FROM
//          bfile_content
//        WHERE
//          file_name = :file_name
//       };
//  ************************************************************

{
  sqlj.runtime.profile.RTResultSet __sJT_rtRs;
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  String __sJT_1 = file_name;
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, LobExample2_SJProfileKeys.getKey(0), 2);
    try 
    {
      __sJT_stmt.setString(1, __sJT_1);
      sqlj.runtime.profile.RTResultSet __sJT_result = __sJT_execCtx.executeQuery();
      __sJT_rtRs = __sJT_result;
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
  try 
  {
    sqlj.runtime.ref.ResultSetIterImpl.checkColumns(__sJT_rtRs, 1);
    if (!__sJT_rtRs.next())
    {
      sqlj.runtime.error.RuntimeRefErrors.raise_NO_ROW_SELECT_INTO();
    }
    host_bfile = ((BFILE)(__sJT_rtRs.getObject(1, BFILE.class)));
    if (__sJT_rtRs.next())
    {
      sqlj.runtime.error.RuntimeRefErrors.raise_MULTI_ROW_SELECT_INTO();
    }
  }
  finally 
  {
    __sJT_rtRs.close();
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:205^5*/

    // step 3: get the file name from the BFILE host object
    String bfile_name = host_bfile.getName();

    // step 4: check that the external file pointed to
    // by the BFILE host object exists
    host_bfile.fileExists();

    // step 5: open the external file via the BFILE host object
    host_bfile.openFile();

    // step 6: create an input stream object to read the external
    // file contents via the BFILE host object
    InputStream input_stream = host_bfile.getBinaryStream();

    // step 7: save the file contents read from the input stream to a new file
    String save_file_name = target_directory + "retrievedBFILE" + bfile_name;
    saveFile(input_stream, save_file_name);

    // step 8: close the input stream
    input_stream.close();

    // step 9: close the external file via the BFILE host object
    host_bfile.closeFile();

    System.out.println("Successfully retrieved BFILE and saved file " +
      save_file_name);

  } // end of retrieveBFILE()


  private static void saveFile(
    InputStream input_stream,
    String file_name
  ) throws IOException {

    // create a file object
    File file = new File(file_name);

    // create a file output stream
    FileOutputStream file_output_stream = new FileOutputStream(file);

    // read the contents of the input stream until
    // the end of the input stream has been reached (indicated
    // by -1 being returned from the read() method)
    byte [] byte_buffer = new byte[8132];
    int bytes_read;
    while ((bytes_read = input_stream.read(byte_buffer)) != -1 ) {

      // write the input to the file
      file_output_stream.write(byte_buffer);

    } // end of while

    // close the file output stream
    file_output_stream.close();

  } // end of saveFile()

}/*@lineinfo:generated-code*/class LobExample2_SJProfileKeys 
{
  private static LobExample2_SJProfileKeys inst = null;
  public static java.lang.Object getKey(int keyNum) 
    throws java.sql.SQLException 
  {
    if (inst == null)
    {
      inst = new LobExample2_SJProfileKeys();
    }
    return inst.keys[keyNum];
  }
  private final sqlj.runtime.profile.Loader loader = sqlj.runtime.RuntimeContext.getRuntime().getLoaderForClass(getClass());
  private java.lang.Object[] keys;
  private LobExample2_SJProfileKeys() 
    throws java.sql.SQLException 
  {
    keys = new java.lang.Object[1];
    keys[0] = sqlj.runtime.ref.DefaultContext.getProfileKey(loader, "LobExample2_SJProfile0");
  }
}
