/*@lineinfo:filename=MultithreadedExample1*//*@lineinfo:user-code*//*@lineinfo:1^1*//*
   The program MultithreadedExample1.sqlj illustrates the use of
   multithreading to update customer names.
*/

import java.sql.*;
import oracle.sqlj.runtime.Oracle;
import sqlj.runtime.ExecutionContext;
import java.util.Random;

public class MultithreadedExample1 extends Thread {

  // declare the iterator class
  /*@lineinfo:generated-code*//*@lineinfo:14^3*/

//  ************************************************************
//  SQLJ iterator declaration:
//  ************************************************************

private static class CustomerIteratorClass 
extends sqlj.runtime.ref.ResultSetIterImpl
implements sqlj.runtime.NamedIterator
{
  public CustomerIteratorClass(sqlj.runtime.profile.RTResultSet resultSet) 
    throws java.sql.SQLException 
  {
    super(resultSet);
    idNdx = findColumn("id");
    first_nameNdx = findColumn("first_name");
    last_nameNdx = findColumn("last_name");
  }
  public int id() 
    throws java.sql.SQLException 
  {
    return resultSet.getIntNoNull(idNdx);
  }
  private int idNdx;
  public String first_name() 
    throws java.sql.SQLException 
  {
    return resultSet.getString(first_nameNdx);
  }
  private int first_nameNdx;
  public String last_name() 
    throws java.sql.SQLException 
  {
    return resultSet.getString(last_nameNdx);
  }
  private int last_nameNdx;
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:16^3*/

  int customer_id;
  String first_name;
  String last_name;

  MultithreadedExample1(
    int customer_id,
    String first_name,
    String last_name
  ) {
    this.customer_id = customer_id;
    this.first_name = first_name;
    this.last_name = last_name;
  }

  public static void main(String [] args) {

    try {

      Oracle.connect(
        "jdbc:oracle:thin:@localhost:1521:orcl",
        "fundamental_user",
        "fundamental_password"
      );

      // display all customers
      displayCustomers();

      // create the thread objects
      MultithreadedExample1 thread1 =
        new MultithreadedExample1(1, "John", "Doe");
      MultithreadedExample1 thread2 =
        new MultithreadedExample1(1, "Jean", "Smith");

      // start the threads using the start() method
      thread1.start();
      thread2.start();

      // wait for each thread to complete using the join() method
      thread1.join();
      thread2.join();

      // display all customers
      displayCustomers();

      /*@lineinfo:generated-code*//*@lineinfo:62^7*/

//  ************************************************************
//  #sql { ROLLBACK  };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, MultithreadedExample1_SJProfileKeys.getKey(0), 0);
    try 
    {
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:62^23*/
      Oracle.close();

    } catch ( SQLException e ) {

      System.err.println("SQLException " + e);
      System.exit(1);

    } catch ( Exception e ) {

      System.err.println("Exception " + e);
      System.exit(1);

    }

  } // end of main()


  public void run() {

    try {

      System.out.println("Updating customer " + customer_id +
        " name to " + first_name + " " + last_name + ".");

      // create new execution context
      ExecutionContext exec_context = new ExecutionContext();

      /*@lineinfo:generated-code*//*@lineinfo:90^7*/

//  ************************************************************
//  #sql [exec_context] { UPDATE
//            customers
//          SET
//            first_name = :first_name,
//            last_name  = :last_name
//          WHERE
//            id = :customer_id
//         };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = exec_context;
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  String __sJT_1 = first_name;
  String __sJT_2 = last_name;
  int __sJT_3 = customer_id;
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, MultithreadedExample1_SJProfileKeys.getKey(0), 1);
    try 
    {
      __sJT_stmt.setString(1, __sJT_1);
      __sJT_stmt.setString(2, __sJT_2);
      __sJT_stmt.setInt(3, __sJT_3);
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:98^7*/

    } catch ( SQLException e ) {

      System.err.println("SQLException " + e);
      System.exit(1);

    } catch ( Exception e ) {

      System.err.println("Exception " + e);
      System.exit(1);

    }

  } // end of run()


  private static void displayCustomers()
  throws SQLException {

    // declare a named iterator object
    CustomerIteratorClass customer_iterator;

    /*@lineinfo:generated-code*//*@lineinfo:121^5*/

//  ************************************************************
//  #sql customer_iterator = { SELECT
//          id, first_name, last_name
//        FROM
//          customers
//        ORDER BY
//          id
//       };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, MultithreadedExample1_SJProfileKeys.getKey(0), 2);
    try 
    {
      sqlj.runtime.profile.RTResultSet __sJT_result = __sJT_execCtx.executeQuery();
      customer_iterator = new CustomerIteratorClass(__sJT_result);
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:128^5*/

    // access the contents of the iterator
    while (customer_iterator.next()) {

      System.out.println("Customer:");
      System.out.println("id = " + customer_iterator.id());
      System.out.println("first_name = " + customer_iterator.first_name());
      System.out.println("last_name = " + customer_iterator.last_name());

    } // end of while loop

    // close the iterator
    customer_iterator.close();

  } // end of displayCustomers()

}/*@lineinfo:generated-code*/class MultithreadedExample1_SJProfileKeys 
{
  private static MultithreadedExample1_SJProfileKeys inst = null;
  public static java.lang.Object getKey(int keyNum) 
    throws java.sql.SQLException 
  {
    if (inst == null)
    {
      inst = new MultithreadedExample1_SJProfileKeys();
    }
    return inst.keys[keyNum];
  }
  private final sqlj.runtime.profile.Loader loader = sqlj.runtime.RuntimeContext.getRuntime().getLoaderForClass(getClass());
  private java.lang.Object[] keys;
  private MultithreadedExample1_SJProfileKeys() 
    throws java.sql.SQLException 
  {
    keys = new java.lang.Object[1];
    keys[0] = sqlj.runtime.ref.DefaultContext.getProfileKey(loader, "MultithreadedExample1_SJProfile0");
  }
}
