/*@lineinfo:filename=ObjectExample1*//*@lineinfo:user-code*//*@lineinfo:1^1*//*
   The program ObjectExample1.sqlj illustrates how to add, retrieve
   and modify column and row objects.
*/

import java.sql.*;
import oracle.sqlj.runtime.Oracle;
import java.math.*;

public class ObjectExample1 {

  /*@lineinfo:generated-code*//*@lineinfo:12^3*/

//  ************************************************************
//  SQLJ iterator declaration:
//  ************************************************************

private static class CustomerIteratorClass 
extends sqlj.runtime.ref.ResultSetIterImpl
implements sqlj.runtime.NamedIterator
{
  public CustomerIteratorClass(sqlj.runtime.profile.RTResultSet resultSet) 
    throws java.sql.SQLException 
  {
    super(resultSet);
    customerNdx = findColumn("customer");
  }
  public TCustomer customer() 
    throws java.sql.SQLException 
  {
    return ((TCustomer)(resultSet.getObject(customerNdx, TCustomer.class)));
  }
  private int customerNdx;
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:12^72*/

  public static void main(String [] args) {

    try {

      Oracle.connect(
        "jdbc:oracle:thin:@localhost:1521:orcl",
        "object_user",
        "object_password"
      );

      int id = 3;
      String first_name = "Jerry";
      String last_name = "Fieldtop";
      Date dob = new Date(80, 11, 2);
      String phone = "650-555-1222";
      String street = "1 Any Street";
      String city = "Any Town";
      String state = "CA";
      String zip = "12345";

      // add a column object to the customers table
      /*@lineinfo:generated-code*//*@lineinfo:35^7*/

//  ************************************************************
//  #sql { INSERT INTO
//            customers (customer)
//          VALUES (
//            t_customer(:id, :first_name, :last_name, :dob, :phone,
//              t_address(:street, :city, :state, :zip)
//            )
//          )
//         };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  int __sJT_1 = id;
  String __sJT_2 = first_name;
  String __sJT_3 = last_name;
  Date __sJT_4 = dob;
  String __sJT_5 = phone;
  String __sJT_6 = street;
  String __sJT_7 = city;
  String __sJT_8 = state;
  String __sJT_9 = zip;
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, ObjectExample1_SJProfileKeys.getKey(0), 0);
    try 
    {
      __sJT_stmt.setInt(1, __sJT_1);
      __sJT_stmt.setString(2, __sJT_2);
      __sJT_stmt.setString(3, __sJT_3);
      __sJT_stmt.setDate(4, __sJT_4);
      __sJT_stmt.setString(5, __sJT_5);
      __sJT_stmt.setString(6, __sJT_6);
      __sJT_stmt.setString(7, __sJT_7);
      __sJT_stmt.setString(8, __sJT_8);
      __sJT_stmt.setString(9, __sJT_9);
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:43^7*/

      // add a row object to the customers2 table
      /*@lineinfo:generated-code*//*@lineinfo:46^7*/

//  ************************************************************
//  #sql { INSERT INTO
//            customers2 (
//              id, first_name, last_name, dob, phone, address
//            ) VALUES (
//              :id, :first_name, :last_name, :dob, :phone,
//                t_address(:street, :city, :state, :zip)
//            )
//         };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  int __sJT_1 = id;
  String __sJT_2 = first_name;
  String __sJT_3 = last_name;
  Date __sJT_4 = dob;
  String __sJT_5 = phone;
  String __sJT_6 = street;
  String __sJT_7 = city;
  String __sJT_8 = state;
  String __sJT_9 = zip;
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, ObjectExample1_SJProfileKeys.getKey(0), 1);
    try 
    {
      __sJT_stmt.setInt(1, __sJT_1);
      __sJT_stmt.setString(2, __sJT_2);
      __sJT_stmt.setString(3, __sJT_3);
      __sJT_stmt.setDate(4, __sJT_4);
      __sJT_stmt.setString(5, __sJT_5);
      __sJT_stmt.setString(6, __sJT_6);
      __sJT_stmt.setString(7, __sJT_7);
      __sJT_stmt.setString(8, __sJT_8);
      __sJT_stmt.setString(9, __sJT_9);
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:54^7*/

      // create a TCustomer host object and use the set mutator
      // methods to initialize the attributes of the host object
      TCustomer customer = new TCustomer();
      id++;
      customer.setId(new BigDecimal(id));
      customer.setFirstName(first_name);
      customer.setLastName(last_name);
      customer.setDob(new Timestamp(dob.getYear(), dob.getMonth(),
        dob.getDay(), 0, 0, 0, 0));
      customer.setPhone(phone);

      // create a TAddress host object and use the set mutator
      // methods to initialize the attributes of the host object
      TAddress address = new TAddress();
      address.setStreet(street);
      address.setCity(city);
      address.setState(state);
      address.setZip(zip);

      // set the address attribute of the customer host object
      customer.setAddress(address);

      // add the customer object to the customers table
      /*@lineinfo:generated-code*//*@lineinfo:79^7*/

//  ************************************************************
//  #sql { INSERT INTO
//            customers (customer)
//          VALUES (
//            :customer
//          )
//         };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  TCustomer __sJT_1 = customer;
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, ObjectExample1_SJProfileKeys.getKey(0), 2);
    try 
    {
      __sJT_stmt.setObject(1, __sJT_1);
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:85^7*/

      // add the customer object to the customers2 table
      /*@lineinfo:generated-code*//*@lineinfo:88^7*/

//  ************************************************************
//  #sql { INSERT INTO
//            customers2
//          VALUES (
//            :customer
//          )
//         };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  TCustomer __sJT_1 = customer;
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, ObjectExample1_SJProfileKeys.getKey(0), 3);
    try 
    {
      __sJT_stmt.setObject(1, __sJT_1);
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:94^7*/

      // update the street attribute in the customers table
      /*@lineinfo:generated-code*//*@lineinfo:97^7*/

//  ************************************************************
//  #sql { UPDATE
//            customers c
//          SET
//            c.customer.address.street = '2 Main Street'
//          WHERE
//            c.customer.id = :id
//         };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  int __sJT_1 = id;
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, ObjectExample1_SJProfileKeys.getKey(0), 4);
    try 
    {
      __sJT_stmt.setInt(1, __sJT_1);
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:104^7*/

      // update the street attribute in the customers2 table
      /*@lineinfo:generated-code*//*@lineinfo:107^7*/

//  ************************************************************
//  #sql { UPDATE
//            customers2 c
//          SET
//            c.address.street = '2 Main Street'
//          WHERE
//            c.id = :id
//         };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  int __sJT_1 = id;
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, ObjectExample1_SJProfileKeys.getKey(0), 5);
    try 
    {
      __sJT_stmt.setInt(1, __sJT_1);
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:114^7*/

      // update the first_name attribute in the customers table
      customer.setFirstName("Jason");
      /*@lineinfo:generated-code*//*@lineinfo:118^7*/

//  ************************************************************
//  #sql { UPDATE
//            customers c
//          SET
//            c.customer = :customer
//          WHERE
//            c.customer.id = :id
//         };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  TCustomer __sJT_1 = customer;
  int __sJT_2 = id;
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, ObjectExample1_SJProfileKeys.getKey(0), 6);
    try 
    {
      __sJT_stmt.setObject(1, __sJT_1);
      __sJT_stmt.setInt(2, __sJT_2);
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:125^7*/

      // update the first_name attribute in the customers2 table
      /*@lineinfo:generated-code*//*@lineinfo:128^7*/

//  ************************************************************
//  #sql { UPDATE
//            customers2 c
//          SET
//            VALUE(c) = :customer
//          WHERE
//            id = :id
//         };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  TCustomer __sJT_1 = customer;
  int __sJT_2 = id;
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, ObjectExample1_SJProfileKeys.getKey(0), 7);
    try 
    {
      __sJT_stmt.setObject(1, __sJT_1);
      __sJT_stmt.setInt(2, __sJT_2);
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:135^7*/

      // use an iterator to retrieve all the objects in the customers table
      CustomerIteratorClass cust_iter;
      /*@lineinfo:generated-code*//*@lineinfo:139^7*/

//  ************************************************************
//  #sql cust_iter = { SELECT
//            c.customer
//          FROM
//            customers c
//          ORDER BY
//            c.customer.id
//         };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, ObjectExample1_SJProfileKeys.getKey(0), 8);
    try 
    {
      sqlj.runtime.profile.RTResultSet __sJT_result = __sJT_execCtx.executeQuery();
      cust_iter = new CustomerIteratorClass(__sJT_result);
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:146^7*/

      // display the customers in the iterator
      System.out.println("The contents of the customers table:");
      while (cust_iter.next()) {
        displayCustomer(cust_iter.customer());
      }

      // close the iterator
      cust_iter.close();

      // retrieve customer #4 from the customers2 table
      /*@lineinfo:generated-code*//*@lineinfo:158^7*/

//  ************************************************************
//  #sql { SELECT
//            VALUE(c)
//          
//          FROM
//            customers2 c
//          WHERE
//            id = :id
//         };
//  ************************************************************

{
  sqlj.runtime.profile.RTResultSet __sJT_rtRs;
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  int __sJT_1 = id;
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, ObjectExample1_SJProfileKeys.getKey(0), 9);
    try 
    {
      __sJT_stmt.setInt(1, __sJT_1);
      sqlj.runtime.profile.RTResultSet __sJT_result = __sJT_execCtx.executeQuery();
      __sJT_rtRs = __sJT_result;
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
  try 
  {
    sqlj.runtime.ref.ResultSetIterImpl.checkColumns(__sJT_rtRs, 1);
    if (!__sJT_rtRs.next())
    {
      sqlj.runtime.error.RuntimeRefErrors.raise_NO_ROW_SELECT_INTO();
    }
    customer = ((TCustomer)(__sJT_rtRs.getObject(1, TCustomer.class)));
    if (__sJT_rtRs.next())
    {
      sqlj.runtime.error.RuntimeRefErrors.raise_MULTI_ROW_SELECT_INTO();
    }
  }
  finally 
  {
    __sJT_rtRs.close();
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:167^7*/

      // display the customer
      System.out.println("Customer #4 from the customers2 table:");
      displayCustomer(customer);

      // rollback the changes and disconnect from the database
      /*@lineinfo:generated-code*//*@lineinfo:174^7*/

//  ************************************************************
//  #sql { ROLLBACK  };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, ObjectExample1_SJProfileKeys.getKey(0), 10);
    try 
    {
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:174^23*/
      Oracle.close();

    } catch (SQLException e) {

      System.err.println("SQLException " + e);
      System.exit(1);

    }
  } // end of main()


  public static void displayCustomer(TCustomer customer)
  throws SQLException {
    System.out.println("id = " + customer.getId());
    System.out.println("first_name = " + customer.getFirstName());
    System.out.println("last_name = " + customer.getLastName());
    System.out.println("dob = " + customer.getDob());
    System.out.println("phone = " + customer.getPhone());
    System.out.println("street = " + customer.getAddress().getStreet());
    System.out.println("city = " + customer.getAddress().getCity());
    System.out.println("state = " + customer.getAddress().getState());
    System.out.println("zip = " + customer.getAddress().getZip());
    System.out.println("age = " + customer.getAge());
  } // end of display_customer()

}/*@lineinfo:generated-code*/class ObjectExample1_SJProfileKeys 
{
  private static ObjectExample1_SJProfileKeys inst = null;
  public static java.lang.Object getKey(int keyNum) 
    throws java.sql.SQLException 
  {
    if (inst == null)
    {
      inst = new ObjectExample1_SJProfileKeys();
    }
    return inst.keys[keyNum];
  }
  private final sqlj.runtime.profile.Loader loader = sqlj.runtime.RuntimeContext.getRuntime().getLoaderForClass(getClass());
  private java.lang.Object[] keys;
  private ObjectExample1_SJProfileKeys() 
    throws java.sql.SQLException 
  {
    keys = new java.lang.Object[1];
    keys[0] = sqlj.runtime.ref.DefaultContext.getProfileKey(loader, "ObjectExample1_SJProfile0");
  }
}
