/*@lineinfo:filename=StreamExample1*//*@lineinfo:user-code*//*@lineinfo:1^1*//*
   The program StreamExample1.sqlj illustrates how to
   use the LONG and LONG RAW types to store text and images
   in the database.
*/

import java.sql.*;
import oracle.sqlj.runtime.Oracle;
import java.io.*;
import sqlj.runtime.*;

public class StreamExample1 {

  public static void main(String [] args) {

    try {

      Oracle.connect(
        "jdbc:oracle:thin:@localhost:1521:orcl",
        "lob_user",
        "lob_password"
      );

      String source_directory = "C:\\sample_files\\";

      addLONG(source_directory, "textContent.txt");
      addLONGRAW(source_directory, "imageContent.jpg");

      Oracle.close();

    } catch (IOException e) {

      System.err.println("IOException " + e);
      System.exit(1);

    } catch (SQLException e) {

      System.err.println("SQLException " + e);
      System.exit(1);

    }

  } // end of main()


  private static void addLONG(
    String source_directory,
    String file_name
  ) throws IOException, SQLException {

    // step 1: create a file object
    File file = new File(source_directory + file_name);

    // step 2: get the length of the file in bytes
    int file_length = (int) file.length();

    // step 3: create a file input stream object to read the file contents
    FileInputStream file_input_stream = new FileInputStream(file);

    // step 4: create an appropriate SQLJ stream host object to store
    // the content
    AsciiStream ascii_stream = new AsciiStream(file_input_stream,
      file_length);

    // step 5: use the SQLJ stream host object to add the content to the
    // database
    /*@lineinfo:generated-code*//*@lineinfo:67^5*/

//  ************************************************************
//  #sql { INSERT INTO
//          long_content (
//            file_name,
//            long_column
//          ) VALUES (
//            :file_name,
//            :ascii_stream
//          )
//       };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  String __sJT_1 = file_name;
  AsciiStream __sJT_2 = ascii_stream;
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, StreamExample1_SJProfileKeys.getKey(0), 0);
    try 
    {
      __sJT_stmt.setString(1, __sJT_1);
      __sJT_stmt.setAsciiStreamWrapper(2, __sJT_2);
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:76^5*/

    // step 6: perform a COMMIT
    /*@lineinfo:generated-code*//*@lineinfo:79^5*/

//  ************************************************************
//  #sql { COMMIT  };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, StreamExample1_SJProfileKeys.getKey(0), 1);
    try 
    {
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:79^19*/

    // step 7: close the SQLJ stream and file input stream objects
    ascii_stream.close();
    file_input_stream.close();

    System.out.println("Successfully added content from file " +
      file_name + " to LONG");

  } // end of addLONG()


  private static void addLONGRAW(
    String source_directory,
    String file_name
  ) throws IOException, SQLException {

    // step 1: create a file object
    File file = new File(source_directory + file_name);

    // step 2: get the length of the file in bytes
    int file_length = (int) file.length();

    // step 3: create a file input stream object to read the file contents
    FileInputStream file_input_stream = new FileInputStream(file);

    // step 4: create an appropriate SQLJ stream host object to store
    // the content
    BinaryStream binary_stream = new BinaryStream(file_input_stream,
      file_length);

    // step 5: use the SQLJ stream host object to add the content to the
    // database
    /*@lineinfo:generated-code*//*@lineinfo:112^5*/

//  ************************************************************
//  #sql { INSERT INTO
//          long_raw_content (
//            file_name,
//            long_raw_column
//          ) VALUES (
//            :file_name,
//            :binary_stream
//          )
//       };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  String __sJT_1 = file_name;
  BinaryStream __sJT_2 = binary_stream;
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, StreamExample1_SJProfileKeys.getKey(0), 2);
    try 
    {
      __sJT_stmt.setString(1, __sJT_1);
      __sJT_stmt.setBinaryStreamWrapper(2, __sJT_2);
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:121^5*/

    // step 6: perform a COMMIT
    /*@lineinfo:generated-code*//*@lineinfo:124^5*/

//  ************************************************************
//  #sql { COMMIT  };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, StreamExample1_SJProfileKeys.getKey(0), 3);
    try 
    {
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:124^19*/

    // step 7: close the SQLJ stream and file input stream objects
    binary_stream.close();
    file_input_stream.close();

    System.out.println("Successfully added content from file " +
      file_name + " to LONG RAW");

  } // end of addLONGRAW()

}/*@lineinfo:generated-code*/class StreamExample1_SJProfileKeys 
{
  private static StreamExample1_SJProfileKeys inst = null;
  public static java.lang.Object getKey(int keyNum) 
    throws java.sql.SQLException 
  {
    if (inst == null)
    {
      inst = new StreamExample1_SJProfileKeys();
    }
    return inst.keys[keyNum];
  }
  private final sqlj.runtime.profile.Loader loader = sqlj.runtime.RuntimeContext.getRuntime().getLoaderForClass(getClass());
  private java.lang.Object[] keys;
  private StreamExample1_SJProfileKeys() 
    throws java.sql.SQLException 
  {
    keys = new java.lang.Object[1];
    keys[0] = sqlj.runtime.ref.DefaultContext.getProfileKey(loader, "StreamExample1_SJProfile0");
  }
}
