/*@lineinfo:filename=StreamExample2*//*@lineinfo:user-code*//*@lineinfo:1^1*//*
   The program StreamExample2.sqlj illustrates how to
   retrieve content from LONG and LONG RAW types.
*/

import java.sql.*;
import oracle.sqlj.runtime.Oracle;
import java.io.*;
import sqlj.runtime.*;

public class StreamExample2 {

  // step 1: declare an iterator class containing an appropriate
  // SQLJ stream object column

  // a named iterator class
  /*@lineinfo:generated-code*//*@lineinfo:17^3*/

//  ************************************************************
//  SQLJ iterator declaration:
//  ************************************************************

private static class LongContentIteratorClass 
extends sqlj.runtime.ref.ResultSetIterImpl
implements sqlj.runtime.NamedIterator
{
  public LongContentIteratorClass(sqlj.runtime.profile.RTResultSet resultSet) 
    throws java.sql.SQLException 
  {
    super(resultSet);
    file_nameNdx = findColumn("file_name");
    long_columnNdx = findColumn("long_column");
  }
  public String file_name() 
    throws java.sql.SQLException 
  {
    return resultSet.getString(file_nameNdx);
  }
  private int file_nameNdx;
  public AsciiStream long_column() 
    throws java.sql.SQLException 
  {
    return resultSet.getAsciiStreamWrapper(long_columnNdx);
  }
  private int long_columnNdx;
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:19^3*/

  // a positional iterator class
  /*@lineinfo:generated-code*//*@lineinfo:22^3*/

//  ************************************************************
//  SQLJ iterator declaration:
//  ************************************************************

private static class LongRawContentIteratorClass 
extends sqlj.runtime.ref.ResultSetIterImpl
implements sqlj.runtime.PositionedIterator
{
  public LongRawContentIteratorClass(sqlj.runtime.profile.RTResultSet resultSet) 
    throws java.sql.SQLException 
  {
    super(resultSet, 2);
  }
  public String getCol1() 
    throws java.sql.SQLException 
  {
    return resultSet.getString(1);
  }
  public BinaryStream getCol2() 
    throws java.sql.SQLException 
  {
    return resultSet.getBinaryStreamWrapper(2);
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:24^3*/

  public static void main(String [] args) {

    try {

      Oracle.connect(
        "jdbc:oracle:thin:@localhost:1521:orcl",
        "lob_user",
        "lob_password"
      );

      String target_directory = "C:\\sample_files\\retrieved\\";

      retrieveLONG(target_directory);
      retrieveLONGRAW(target_directory);
      retrieveLONGplsql("textContent.txt", target_directory);

      Oracle.close();

    } catch (IOException e) {

      System.err.println("IOException " + e);
      System.exit(1);

    } catch (SQLException e) {

      System.err.println("SQLException " + e);
      System.exit(1);

    }

  } // end of main()


  private static void retrieveLONG(
    String target_directory
  ) throws IOException, SQLException {

    // step 2: declare an iterator object from the iterator class
    LongContentIteratorClass long_content_iterator;

    // step 3: declare an appropriate SQLJ stream object to store
    // the content from the iterator SQLJ stream object column
    AsciiStream ascii_stream = null;

    // step 4: populate the iterator object using a database query
    /*@lineinfo:generated-code*//*@lineinfo:71^5*/

//  ************************************************************
//  #sql long_content_iterator = { SELECT
//          file_name,
//          long_column
//        FROM
//          long_content
//       };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, StreamExample2_SJProfileKeys.getKey(0), 0);
    try 
    {
      sqlj.runtime.profile.RTResultSet __sJT_result = __sJT_execCtx.executeQuery();
      long_content_iterator = new LongContentIteratorClass(__sJT_result);
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:77^5*/

    // step 5: retrieve the contents of the iterator object
    while (long_content_iterator.next()) {

      String file_name = long_content_iterator.file_name();
      String save_file_name = target_directory + "retrievedLONG" +
        file_name;

      // retrieve the SQLJ stream object
      ascii_stream = long_content_iterator.long_column();

      // step 6: save the SQLJ stream object to a new file
      saveFile(ascii_stream, save_file_name);

      System.out.println("Successfully retrieved LONG and saved file " +
        save_file_name);

      // step 7: close the SQLJ stream object
      ascii_stream.close();

    } // end of while

    // step 8: close the iterator object
    long_content_iterator.close();

  } // end of retrieveLONG()


  private static void retrieveLONGRAW(
    String target_directory
  ) throws IOException, SQLException {

    // step 2: declare an iterator object from the iterator class
    LongRawContentIteratorClass long_raw_content_iterator;

    // step 3: declare an appropriate SQLJ stream object to store
    // the content from the iterator SQLJ stream object column
    BinaryStream binary_stream = null;

    // step 4: populate the iterator object using a database query
    /*@lineinfo:generated-code*//*@lineinfo:118^5*/

//  ************************************************************
//  #sql long_raw_content_iterator = { SELECT
//          file_name,
//          long_raw_column
//        FROM
//          long_raw_content
//       };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, StreamExample2_SJProfileKeys.getKey(0), 1);
    try 
    {
      sqlj.runtime.profile.RTResultSet __sJT_result = __sJT_execCtx.executeQuery();
      long_raw_content_iterator = new LongRawContentIteratorClass(__sJT_result);
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:124^5*/

    // step 5: retrieve the contents of the iterator object
    boolean finished = false;
    String file_name = null;
    while (!finished) {

      /*@lineinfo:generated-code*//*@lineinfo:131^7*/

//  ************************************************************
//  #sql { FETCH :long_raw_content_iterator
//          
//         };
//  ************************************************************

{
  LongRawContentIteratorClass __sJT_iter = long_raw_content_iterator;
  if (__sJT_iter.next())
  {
    file_name = __sJT_iter.getCol1();
    binary_stream = __sJT_iter.getCol2();
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:134^7*/

      // if the end of rows has been reached, set finished to true
      if (long_raw_content_iterator.endFetch()) {

        finished = true;

      } else {

        String save_file_name = target_directory + "retrievedLONGRAW" +
          file_name;

        // step 6: save the SQLJ stream object to a new file
        saveFile(binary_stream, save_file_name);

        System.out.println("Successfully retrieved LONG RAW and " +
          "saved file " + save_file_name);

        // step 7: close the SQLJ stream object created in step 3
        binary_stream.close();

      } // end if
        
    } // end of while

    // step 8: close the iterator object
    long_raw_content_iterator.close();

  } // end of retrieveLONGRAW()


  private static void retrieveLONGplsql(
    String file_name,
    String target_directory
  ) throws IOException, SQLException {

    // step 1: declare an appropriate SQLJ stream object to store
    // the stream returned from the PL/SQL function
    AsciiStream ascii_stream;

    // step 2: call the PL/SQL function using the VALUES statement
    // and store the returned variable in the SQLJ stream object
    // created in step 1
    /*@lineinfo:generated-code*//*@lineinfo:177^5*/

//  ************************************************************
//  #sql ascii_stream = { VALUES(retrieve_LONG_func(:file_name))  };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  String __sJT_2 = file_name;
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, StreamExample2_SJProfileKeys.getKey(0), 2);
    try 
    {
      __sJT_stmt.setString(2, __sJT_2);
      __sJT_execCtx.executeUpdate();
      AsciiStream __sJT_result;
      __sJT_result = ((AsciiStream)(__sJT_stmt.getObject(1, AsciiStream.class)));
      ascii_stream = __sJT_result;
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:177^66*/

    // step 3: save the SQLJ stream object to a new file
    String save_file_name = target_directory + "retrievedLONGplsql" +
      file_name;
    saveFile(ascii_stream, save_file_name);

    System.out.println("Successfully retrieved LONG and saved file " +
      save_file_name);

    // step 4: close the SQLJ stream object
    ascii_stream.close();

  } // end of retrieveLONGplsql()


  private static void saveFile(
    InputStream input_stream,
    String file_name
  ) throws IOException {

    // create a file object
    File file = new File(file_name);

    // create a file output stream
    FileOutputStream file_output_stream = new FileOutputStream(file);

    // read the contents of the input stream until
    // the end of the input stream has been reached (indicated
    // by -1 being returned from the read() method)
    byte [] byte_buffer = new byte[8132];
    int bytes_read;
    while ((bytes_read = input_stream.read(byte_buffer)) != -1 ){

      // write the input to the file
      file_output_stream.write(byte_buffer);

    } // end of while

    // close the file output stream
    file_output_stream.close();

  } // end of saveFile()

}/*@lineinfo:generated-code*/class StreamExample2_SJProfileKeys 
{
  private static StreamExample2_SJProfileKeys inst = null;
  public static java.lang.Object getKey(int keyNum) 
    throws java.sql.SQLException 
  {
    if (inst == null)
    {
      inst = new StreamExample2_SJProfileKeys();
    }
    return inst.keys[keyNum];
  }
  private final sqlj.runtime.profile.Loader loader = sqlj.runtime.RuntimeContext.getRuntime().getLoaderForClass(getClass());
  private java.lang.Object[] keys;
  private StreamExample2_SJProfileKeys() 
    throws java.sql.SQLException 
  {
    keys = new java.lang.Object[1];
    keys[0] = sqlj.runtime.ref.DefaultContext.getProfileKey(loader, "StreamExample2_SJProfile0");
  }
}
