-- The SQL*Plus script server_examples.sql performs the following:
--   Creates the various wrappers, procedures and triggers for the
--   Deployment chapter.

-- connect as the fundamental_user
CONNECT fundamental_user/fundamental_password;

-- create a procedure for the ServerExample1.testMessage() method
CREATE OR REPLACE PROCEDURE test_message
AS
LANGUAGE JAVA
NAME 'ServerExample1.testMessage()';
/

-- create a procedure for the ServerExample1.addCustomer() method
CREATE OR REPLACE PROCEDURE add_customer(
  first_name VARCHAR2,
  last_name  VARCHAR2
) AS
LANGUAGE JAVA
NAME 'ServerExample1.addCustomer(java.lang.String, java.lang.String)';
/

-- create a procedure for the ServerExample1.displayCustomer() method
CREATE OR REPLACE PROCEDURE display_customer(
  id NUMBER
) AS
LANGUAGE JAVA
NAME 'ServerExample1.displayCustomer(int)';
/

-- create a function for the ServerExample1.countCustomers() method
CREATE OR REPLACE FUNCTION count_customers
RETURN NUMBER AS
LANGUAGE JAVA
NAME 'ServerExample1.countCustomers() return int';
/


-- create a package named server_example1 for
-- the methods in the ServerExample1 class
CREATE OR REPLACE PACKAGE server_example1 AS

  PROCEDURE test_message;

  PROCEDURE add_customer (
    first_name VARCHAR2,
    last_name  VARCHAR2
  );

  PROCEDURE display_customer (
    id NUMBER
  );

  FUNCTION count_customers
  RETURN NUMBER;

END server_example1;
/

-- create a package body for server_example1
CREATE OR REPLACE PACKAGE BODY server_example1 AS

  PROCEDURE test_message
  AS
  LANGUAGE JAVA
  NAME 'ServerExample1.testMessage()';

  PROCEDURE add_customer (
    first_name  VARCHAR2,
    last_name   VARCHAR2
  ) AS
  LANGUAGE JAVA
  NAME 'ServerExample1.addCustomer(java.lang.String, java.lang.String)';
  
  PROCEDURE display_customer (
    id NUMBER
  ) AS
  LANGUAGE JAVA
  NAME 'ServerExample1.displayCustomer(int)';

  FUNCTION count_customers
  RETURN NUMBER AS
  LANGUAGE JAVA
  NAME 'ServerExample1.countCustomers() return int';

END server_example1;
/


-- create a procedure named add_display_count that
-- calls the wrappers in the package server_example1
-- to add and display a new customer, and count the number
-- of customers
CREATE OR REPLACE PROCEDURE add_display_count AS

  number_of_customers INT;

BEGIN

  server_example1.add_customer('Joe', 'Smith');
  server_example1.display_customer(8);
  number_of_customers := server_example1.count_customers;
  dbms_output.put_line('There are ' || number_of_customers ||
    ' customers');

END add_display_count;
/


-- create a database trigger named price_update_trigger
-- that fires when the new price for a row in the products table
-- is 10% greater than the old price.
-- The trigger audits the event by storing the product id,
-- the new price and the old price in the price_audit table.
CREATE OR REPLACE TRIGGER
  product_price_update_trigger
BEFORE UPDATE OF
  price
ON
  products
FOR EACH ROW
WHEN
  (new.price > old.price * 1.1)
BEGIN

  dbms_output.put_line('For product id ' || :old.id);
  dbms_output.put_line('Old price = ' || :old.price);
  dbms_output.put_line('New price = ' || :new.price);
  dbms_output.put_line('The new price is 10% greater than the old price - auditing change');

  -- insert row into the price_audit table
  INSERT INTO
    price_audit
  VALUES (
    :old.id, :old.price, :new.price
  );

END;
/


CREATE OR REPLACE PROCEDURE product_price_update(
  product_id NUMBER,
  old_price  NUMBER,
  new_price  NUMBER
) AS
LANGUAGE JAVA
NAME 'ServerExample2.productPriceUpdate(int, double, double)';
/


-- recreate the database trigger named price_update_trigger
--
-- The body of the trigger has been rewritten in SQLJ and
-- moved to productPriceUpdate() in the file ServerExample2.sqlj.
CREATE OR REPLACE TRIGGER
  product_price_update_trigger
BEFORE UPDATE OF
  price
ON
  products
FOR EACH ROW
WHEN
  (new.price > old.price * 1.1)
CALL product_price_update(:old.id, :old.price, :new.price)
/
